#! /usr/bin/perl
#
# Copyright (C) 2012-2021 Alexis Bienvenüe <paamc@passoire.fr>
#
# This file is part of Auto-Multiple-Choice
#
# Auto-Multiple-Choice is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 2 of
# the License, or (at your option) any later version.
#
# Auto-Multiple-Choice is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Auto-Multiple-Choice.  If not, see
# <http://www.gnu.org/licenses/>.

require "./AMC/Test.pm";

AMC::Test->new(
    dir             => __FILE__,
    filter          => 'plain',
    tex_engine      => 'xelatex',
    seuil           => 0.15,
    export_full_csv => [
        { -copy => 4, -question => 'Cam',      -score => 1 },
        { -copy => 4, -question => 'positive', -score => 2 },
        { -copy => 4, -question => 'one',      -score => 0 },
        { -copy => 4, -question => 'eleven',   -score => 1 },
        { -copy => 4, -question => 'twelve',   -score => 1 },
        { -copy => 4, -question => 'zero',     -score => 4 },
        { -copy => 4, -question => 'way',      -score => 2 },
    ],
)->default_process;

